#!/usr/bin/env bash
#
# Copyright (C) 2003 VA Linux Systems Japan, K.K.
#
# LICENSE NOTICE
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#

# $Id: slapd.sh,v 1.5 2004/04/12 04:35:49 taru Exp $

set -e

eval `ultrapossum-config init`

tmp=`tempfile`
trap "/bin/rm -f $tmp; eval `ultrapossum-config term`" 0

common() {

  INDEXES_EQ="objectClass"
  if test "x$SLAPD_PROVIDES_SYNCBACKUP" != "x"; then
    INDEXES_EQ="$INDEXES_EQ,contextCSN,entryUUID"
  fi
  ## FIXME: handle database specific grammer?
  ## TODO: choose apps
  for cf in `/bin/ls $APPSDIR`
  do
    . $APPSDIR/$cf
    if test -f $APPSCONFDIR/$cf.cf; then
      . $APPSCONFDIR/$cf.cf
    fi
    for i in $indexes_eq
    do
      if ! echo "$INDEXES_EQ" | grep $i > /dev/null; then
        INDEXES_EQ="$INDEXES_EQ
$i"
      fi
    done

    for i in $indexes_sub
    do
      if test "x$INDEXES_SUB" = "x"; then
        INDEXES_SUB=$i
      else
        if ! echo "$INDEXES_SUB" | grep $i > /dev/null; then
          INDEXES_SUB="$INDEXES_SUB
$i"
        fi
      fi
    done

  done

  INDEXES_EQ=`echo -n "$INDEXES_EQ" | tr '\n' ','`
  INDEXES_SUB=`echo -n "$INDEXES_SUB" | tr '\n' ','`
  test -z "$INDEXES_EQ"  || INDEXES="index $INDEXES_EQ eq"
  test -z "$INDEXES_SUB" || INDEXES="$INDEXES\\
index $INDEXES_SUB sub"

  SLAPROOTPW_ESCAPE=`echo $SLAPROOTPW | sed 's!/!\\\\/!g'`
  sed -e "s/#SUFFIX#/$SUFFIX/" -e "s/#ROOTDN#/$ROOTDN/" \
  	-e "s/#SLAPROOTPW#/$SLAPROOTPW_ESCAPE/" -e "s!#DIRECTORY#!$DIRECTORY!" \
  	-e "s/#DBCACHESIZE#/$DBCACHESIZE/" -e "s/#APPLICATION#/$APPLICATION/" \
  	-e "s/#UPDATEDN#/$UPDATEDN/" -e "/#INDEXES#/ c\\" -e "$INDEXES" \
  	-e "s!#REPLOGFILE#!$REPLOGFILE!" -e "s!#CACHESIZE#!$CACHESIZE!"
}

master() {
  common
  for slave in $SLURPDSLAVES
  do
    echo "replica host=$slave:$REPLICALISTENPORT binddn=\"$UPDATEDN\""
    echo "	bindmethod=simple"
    echo "	credentials=$CREDENTIALS"
    echo ""
  done
  if test "x$ROOTDN" = "x$UPDATEDN"; then
    echo "# Replication"
    echo "updatedn \"$UPDATEDN\""
    if test "x$UPDATEREF" != "x"; then
      echo "updateref $UPDATEREF"
    fi
  fi
  if test "x$SYNCBACKUPS" != "x"; then
    echo "syncdn $SYNCDN"
    if test "x$WEAKSYNC" != "x"; then
      echo "weaksync on"
    fi
    for slave in $SYNCBACKUPS
    do
      if test "x$HOST" != "x$slave"; then
        echo "replica host=$slave"
        echo "	binddn=\"$UPDATEDN\""
        echo "	syncid=$slave"
        echo "	bindmethod=simple"
        echo "	credentials=$CREDENTIALS"
        echo ""
      fi
    done
  fi
}

slave() {
  common
  if test "x$SLAVES$ANOTHER_MASTER" != "x" || include "$HOST" "$SYNCBACKUPS"; then
    echo "# Replication"
    echo "updatedn \"$UPDATEDN\""
    if test "x$UPDATEREF" != "x"; then
      echo "updateref $UPDATEREF"
    fi
  fi
  if include "$HOST" "$SYNCBACKUPS"; then
    echo "syncbackup syncid=$HOST"
    echo "	provider=$UPDATEREF"
    echo "	binddn=$SYNCDN"
    echo "	bindmethod=simple"
    echo "	credentials=$SYNCPW"
    echo "	checkinterval=10"
  fi
}

if test "x$2" = "x"; then
  echo "Usage: $0 <slapd.conf.in> <master|slave>"
  exit 1
fi

cat $1 | $2

